require 'spec_helper'

module Hikki
  module Adapters
    describe MemcacheAdapter do
      subject(:adapter) { MemcacheAdapter.new(connection, uuid_generator) }
      let(:connection) { double :connection, set: true, get: nil, delete: true, flush: true }
      let(:uuid_generator) { double :uuid_generator, uuid: '12345' }
      let(:collection) { 'collection1' }

      describe '#index' do
        it 'returns false' do
          expect(adapter.index(collection, :field1)).to be_false
        end
      end

      describe '#save' do
        context 'when an id is provided in the data' do
          let(:data) { { id: id, field1: 'test', field2: 123 } }
          let(:expected) { { 'id' => id, 'field1' => 'test', 'field2' => 123 } }
          let(:id) { '1' }
          let(:key) { collection + ':' + id }

          it 'returns the data' do
            expect(adapter.save(collection, data)).to eq expected
          end

          it 'persists the data in the store' do
            adapter.save(collection, data)
            expect(connection).to have_received(:set).with(key, expected)
          end
        end

        context 'when an id is not provided in the data' do
          let(:data) { { field1: 'test', field2: 123 } }
          let(:expected) { { 'id' => id, 'field1' => 'test', 'field2' => 123 } }
          let(:id) { '12345' }
          let(:key) { collection + ':' + id }

          it 'returns the data with the id added' do
            expect(adapter.save(collection, data)).to eq expected
          end

          it 'persists the data in the store' do
            adapter.save(collection, data)
            expect(connection).to have_received(:set).with(key, expected)
          end
        end
      end

      describe '#find' do
        let(:id) { '1' }
        let(:key) { collection + ':' + id }

        context 'when the id exists' do
          let(:expected) { { 'id' => id, 'field1' => 'test', 'field2' => 123 } }
          before { connection.stub(:get).with(key).and_return(expected) }

          it 'retrieves the data' do
            expect(adapter.find(collection, id)).to eq expected
          end
        end

        context 'when the id does not exist' do
          let(:expected) { {} }
          before { connection.stub(:get).with(key).and_return(nil) }

          it 'returns an empty hash' do
            expect(adapter.find(collection, id)).to eq expected
          end
        end
      end

      describe '#all' do
        it 'returns []' do
          expect(adapter.all(collection)).to eq []
          expect(adapter.all(collection, {})).to eq []
        end
      end

      describe '#find_by' do
        it 'returns []' do
          expect(adapter.find_by(collection, :field1, 'test')).to eq []
          expect(adapter.find_by(collection, :field1, 'test', {})).to eq []
        end
      end

      describe '#remove' do
        let(:id) { '1' }
        let(:key) { collection + ':' + id }

        it 'returns true' do
          expect(adapter.remove(collection, id)).to be_true
        end

        it 'removes the record from the store' do
          adapter.remove(collection, id)
          expect(connection).to have_received(:delete).with(key)
        end
      end

      describe '#remove_all' do
        it 'returns true' do
          expect(adapter.remove_all(collection)).to be_true
        end

        it 'removes all of the records from the store' do
          adapter.remove_all(collection)
          expect(connection).to have_received(:flush)
        end
      end

    end
  end
end